/* 
 *  1987     john h. dubois iii (john@armory.com)
 *  90/11/01 ported to sysv
 *  91/09/28 modded to work with 8-char logins (changed strcmp to strncmp)
 *  91/11/27 Behave more like tty if no login name given: print name of
 *	     tty associated with stdin, instead of searching for username
 *	     in utmp.
 *  91/11/29 added (char *) cast in fread() to eliminate warning from XENIX
 *	     devsys.
 *  96/07/03 Added help option.
 */
#ifdef _M_SYSV
#define M_SYS5
#endif
#ifdef M_SYSV
#define M_SYS5
#endif
#ifdef M_SYS5
#	include <sys/types.h>
#	include <string.h>
#	include <pwd.h>
#else
#	include <strings.h>
#	define strchr index
#endif

#include <utmp.h>
#include <stdio.h>
#define ExTest(Test,Mess) if (Test) {fputs(Mess,stderr); exit(2);}
#define TRUE 1
#define FALSE 0

char *Comment = "@(#) utty 1.2 96/07/03";

main(argc,argv)
int argc;
char **argv;
{
#ifdef M_SYS5
	struct passwd *PwEnt;
	char *tty;
	char *ttyname();
#endif
	struct utmp Utmp;	/* Struct read from /etc/utmp */
	FILE *InFile;	/* Opened utmp file */
	char *getlogin();
	char **Flags;	/* Flags found on command line */
	char **Users;	/* User names to search for */
	char *Name;	/* tty name to be printed */
	int NumFlags;	/* Number of flags found on command line */
	int NumUsers;	/* Number of users to find the tty for */
	int i;
	int UserFound = FALSE;	/* Whether at least one user was found */
	int ShortName = FALSE;	/* Print tty name without leading "tty" part */
	int One = FALSE;    /* Print only the first tty found for each user*/
	int Brief = FALSE;	/* Don't print user names, only tty names */

	NumFlags = SepFlags(argc,argv,"-",&Flags,&Users);
	NumUsers = argc - NumFlags - 1;
	for (i = 0; i < NumFlags; i++)
		for (; *Flags[i]; Flags[i]++)
			switch (*Flags[i]) {
				case 's' :	ShortName = TRUE;
							break;
				case '1' :	One = TRUE;
							break;
				case 'b' :	Brief = TRUE;
							break;
				case 'h' :	fputs(
		"utty: Show what tty a user is on.\n"\
		"Syntax:  utty [-hs1b] [user-name] ...\n"\
		"-s: Short names; remove leading 'tty' from device names.\n"\
		"-1: Print only the first tty the user is on.\n"\
		"-b: Brief; do not print user name.\n",stdout);
							exit(0);
				default :	fputs(
		"utty: Bad flag.  Use -h for help.\n",stderr);
							exit(3);
			}

	/* If no user names given on command line, 
	   print tty for user running program (like tty does) */
	if (!NumUsers) {
#ifdef M_SYS5
		PwEnt = getpwuid(getuid());
		ExTest(!PwEnt,"Could not get login name from passwd file.\n");
		tty = ttyname(0);
		ExTest(!tty,"stdin is not a tty.\n");
		PrintName(tty + 5,PwEnt->pw_name,ShortName,Brief);
		exit(0);
#else
		NumUsers = 1;
		Users = (char **) getlogin();
		ExTest(!Users,"Could not get login name.\n");
#endif
	}
	ExTest(!(InFile = fopen("/etc/utmp","r")),
	 "Could not open /etc/utmp.\n");
	while (!feof(InFile)) {
		fread((char *) &Utmp,sizeof(Utmp),1,InFile);
		for (i = 0; i < NumUsers; i++)
			if (!strncmp(Utmp.ut_name,Users[i],8)) {
				UserFound = TRUE;
				PrintName(Utmp.ut_line,Users[i],ShortName,Brief);
				if (One)
					exit(0);
			}
	}
	if (!UserFound)
		exit(1);
}

PrintName(Name,User,ShortName,Brief)
char *Name;
char *User;
int ShortName;
int Brief;
{
	if (ShortName)
		if (*Name == 't')
			Name += 3;
		else	/* print "co" for console */
			Name[2] = '\0';
	if (!Brief) {
		fputs(User,stdout);
		fputs(": ",stdout);
	}
	puts(Name);
}

int SepFlags(argc,argv,FlagSpecs,Flags,Files)
int argc;
char *argv[],*FlagSpecs,***Flags,***Files;
/* Separates argv into flags and filenames.  Flags and Files are malloced to
   be arrays of (argc) char pointers.  Pointers to flags (arguments that begin
   with one of the characters in FlagSpecs) are put in Flags,with the flag
   specifier removed from the front; pointers to all other argument are copied
   to Files.  The pointer in Flags and Files following the last pointer copied
   is set to NULL. The number of flags found is returned.  argv[0] is ignored.
*/
{
   int I;
   char **FlgTmp,**FilTmp;	/* Temporary flag and filename array ptrs */
   int NumFlgs = 0,NumFls = 0;	/* Number of flags and filenames found */

   FlgTmp = (char **) calloc(argc,sizeof(char *));
   FilTmp = (char **) calloc(argc,sizeof(char *));
   for (I = 1; I < argc; I++)	/* For each argument */
      if (strchr(FlagSpecs,*argv[I]) == NULL)
         FilTmp[NumFls++] = argv[I];
      else
         FlgTmp[NumFlgs++] = argv[I] + 1;
   FilTmp[NumFls] = NULL;
   FlgTmp[NumFlgs] = NULL;
   *Flags = FlgTmp;	/* Copy temp pointers to arrays to parameters */
   *Files = FilTmp;
   return(NumFlgs);
}
